﻿# Config Guide for RRR Monsters and RRR Npcs Custom Mob JSON
#
### File Management
 - JSON files must be anywhere in your `Valheim\BepInEx\config` folder. Folders inside of that folder will also be loaded, so you can use subfolders for organization.
 - To be loaded, JSON files must follow the format `rrrnpcs.custom.*.json` or `rrrmonsters.custom.*.json`.
   
### Getting Started
 - ####***To get started you need a template file to work with. Look at the following two sections depending on whether you want a Monster or an NPC.***
#### Monster Templates
 - In `com.alexanderstrada.rrrmonsters.cfg` fill `PrefabNames` with a list of Monsters you want templates for, such as "`Troll, Goblin, Serpent, Greyling`".
 - Load into a Valheim world and the templates will be created in your config folder. 
 - Monsters with the name format `rrrmonsters.custom.Exported*.json` will not be loaded. Rename them after you export.
#### NPC Templates
 - Each time you load a Valheim world with RRR Npcs installed it will save an NPC template to `rrrnpcs.custom.Example.json`, this template will not be loaded. Rename before editing or it will be overwritten.
---

 #

---

##General Info
 - Config files use the JSON format. It should be intuitive for new users but if you find yourself confused, any internet resource on JSON can help you here.
 - A config file defines an existing mob that your prefab is based on (for monsters, something like "Troll" or "Greyling", for NPCs it is always "RRR_NPC".)
--- 
 - The config file is broken up into category sections. Any category can be removed (set to null or left unspecified) to have the settings in that category revert to the default for the base prefab.
---
 - Fields in each category are named with a lowercase prefix that tells you what sort of data goes in that field. Current data types are:

| Prefix | Type    | Description                                                                                                                                                                          | Example                                                                                                                                                                                                                                                 |
|--------|---------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| s      | string  | Text, "surrounded by quotes"                                                                                                                                                         | `"sOriginalPrefabName" : "RRR_NPC"`                                                                                                                                                                                                                     |
| b      | bool    | A switch, `true` or `false`                                                                                                                                                          | `"bToleratesWater" : true`                                                                                                                                                                                                                              |
| f      | float   | Number that can have decimals                                                                                                                                                        | `"fHairValueMin" : 0.1`                                                                                                                                                                                                                                 |
| i      | int     | Number that cannot have decimals                                                                                                                                                     | `"iAmountMin" : 20`                                                                                                                                                                                                                                     |
| v      | vector  | Object with 3 floats representing spacial dimensions (X, Y, Z)                                                                                                                       | `"vScale" : {"fX" : 1, "fY" : 1, "fZ" : 1}`                                                                                                                                                                                                             |
| c      | color   | Object with 3 floats [0.0-1.0] representing RGB color (R, G, B)                                                                                                                      | `"cHairColorMin" : {"fRed" : 0.0, "fGreen" : 0.5,"fBlue" : 1.0}`                                                                                                                                                                                        |
| a      | array   | A list of strings, comma separated, each in quotes.                                                                                                                                  | `"aBeardIds" : ["Beard1", "Beard2", "Beard3"]`                                                                                                                                                                                                          |
| aa     | array^2 | A list of arrays, comma separated, each in quotes.                                                                                                                                   | `"aaSetItems" : [{"aSetItems" : ["troll_punch", "troll_throw"]},{"aSetItems" : ["troll_log_swing_v"]}]`                                                                                                                                                 |
| d      | defense | A damage taken object, defines your mob's weaknesses and resistances. Valid values are `"Normal"`, `"Resistant"`, `"Weak"`, `"Immune"`, `"Ignore"`, `"VeryResistant"`, `"VeryWeak"`. | `"dDamageTaken" : {"m_blunt" : "Resistant", "m_slash" : "Normal", "m_pierce" : "Weak", "m_chop" : "Ignore", "m_pickaxe" : "Ignore", "m_fire" : "Normal", "m_frost" : "Normal", "m_lightning" : "Normal", "m_poison" : "Normal", "m_spirit" : "Immune"}` |
| dt     | damage  | A damage dealt object, defines your mob's attack damage by type in floats. Can be `null` to use the base mob's damage instead.                                                       | `"dtAttackDamageOverride" : {"fDamage" : 0, "fBlunt" : 0, "fSlash" : 0, "fPierce" : 0, "fChop" : 0, "fPickaxe" : 0, "fFire" : 0, "fFrost" : 0, "fLightning" : 0, "fPoison" : 0, "fSpirit" : 0}`                                                         |
| dr     | drops   | A list of Drop objects, defining what items your mob drops when it dies.                                                                                                             | `"dDrops" : [{"sPrefabName" : "Coins", "iAmountMin" : 20, "iAmountMax" : 30, "fChance" : 1, "bOnePerPlayer" : false, "bLevelMultiplier" : true}]`                                                                                                       |

---

# Settings Guide

|      **CATEGORY**     |                      **TAG**                     |                                                                                                                                                                **DESCRIPTION**                                                                                                                                                               |                                                                                                               **EXAMPLE**                                                                                                              |
|:---------------------:|:------------------------------------------------:|:--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------:|:--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------:|
|         (none)        |               "sOriginalPrefabName"              |                                                                                                                                       The name of the original prefab your custom variant is based on.                                                                                                                                       |                                                                                                                 "Troll"                                                                                                                |
|         (none)        |                 "sNewPrefabName"                 |                                                                                 A name for your new custom prefab. If this is "Jimmy", the  full prefab name you use in other mods or the game console will be "RRRN_Jimmy" if NPC, "RRRM_Jimmy" if Monster.                                                                                 |                                                                                                                "MyTroll"                                                                                                               |
|                       |                                                  |                                                                                                                                                                                                                                                                                                                                              |                                                                                                                                                                                                                                        |
| "Category_Appearance" |                     "vScale"                     |                                                                                                                                Used to change the size/shape of your custom mob. X/Y/Z is Width/Height/Depth.                                                                                                                                |                                                                                                  {"fX" : 0.9, "fY" : 1.2, "fZ" : 0.9}                                                                                                  |
| "Category_Appearance" |                   "cTintColor"                   |                                                                                                                               Applies a color tinting to your custom mob. Values should be between 0.0 and 1.0                                                                                                                               |                                                                                              {"fRed" : 0.0, "fGreen" : 0.5,"fBlue" : 1.0}                                                                                              |
| "Category_Appearance" |                   "cBodySmoke"                   |                                                                                                                      Applies a full-body colored smoke particle effect to your mob. Values should be between 0.0 and 1.0                                                                                                                     |                                                                                              {"fRed" : 0.0, "fGreen" : 0.5,"fBlue" : 1.0}                                                                                              |
| "Category_Appearance" |                   "cBodyFlames"                  |                                                                                                                     Applies a full-body colored flames particle effect to your mob. Values should be between 0.0 and 1.0                                                                                                                     |                                                                                              {"fRed" : 0.0, "fGreen" : 0.5,"fBlue" : 1.0}                                                                                              |
|                       |                                                  |                                                                                                                                                                                                                                                                                                                                              |                                                                                                                                                                                                                                        |
|  "Category_Character" |                      "sName"                     |                                                                                                                                          The display name for your mob that appears over their head.                                                                                                                                         |                                                                                                         "Jimmy the Happy Troll"                                                                                                        |
|  "Category_Character" |                    "sFaction"                    |                                                              Your mob's faction, determines who they will fight with. Valid Entries: `"Players"`, `"AnimalsVeg"`, `"ForestMonsters"`, `"Undead"`, `"Demon"`, `"MountainMonsters"`, `"SeaMonsters"`, `"PlainsMonsters"`, `"Boss"`                                                             |                                                                                                            "ForestMonsters"                                                                                                            |
|  "Category_Character" |                      "bBoss"                     |                                                                                                                                                      Whether or not this mob is a Boss.                                                                                                                                                      |                                                                                                                  false                                                                                                                 |
|  "Category_Character" |                   "sBossEvent"                   |                                                                                           These are used to define the environmental effects of boss fights, but only seems to work when bosses are spawned "correctly" by boss altars/sacrifices.                                                                                           |                                                                                                              "boss_gdking"                                                                                                             |
|  "Category_Character" |               "sDefeatSetGlobalKey"              |                                                                                                                                When this mob dies, sets this global key. Keys are used by raids and some mods.                                                                                                                               |                                                                                                            "defeated_gdking"                                                                                                           |
|  "Category_Character" |                 "fMoveSpeedMulti"                |                                                                                                                                                  A multiplier for this mob's movement speed.                                                                                                                                                 |                                                                                                                   2.0                                                                                                                  |
|  "Category_Character" |                 "bToleratesWater"                |                                                                                                                                                If false, this mob will take damage from water.                                                                                                                                               |                                                                                                                  true                                                                                                                  |
|  "Category_Character" |                 "bToleratesFire"                 |                                                                                                                                                If false, this mob will take damage from fire.                                                                                                                                                |                                                                                                                  false                                                                                                                 |
|  "Category_Character" |                 "bToleratesSmoke"                |                                                                                                                                                If false, this mob will take damage from smoke.                                                                                                                                               |                                                                                                                  false                                                                                                                 |
|  "Category_Character" |                     "fHealth"                    |                                                                                                                                                        Your mob's health point total.                                                                                                                                                        |                                                                                                                   400                                                                                                                  |
|  "Category_Character" |                  "dDamageTaken"                  |                                                                                  Defines your mob's weaknesses and resistances by damage type. Valid entries:  `"Normal"`, `"Resistant"`, `"Weak"`, `"Immune"`, `"Ignore"`, `"VeryResistant"`, `"VeryWeak"`.                                                                                 | {"m_blunt" : "Resistant",  "m_slash" : "Normal",  "m_pierce" : "Weak", "m_chop" : "Ignore", "m_pickaxe" : "Ignore", "m_fire" : "Normal", "m_frost" : "Normal", "m_lightning" : "Normal", "m_poison" : "Normal", "m_spirit" : "Immune"} |
|                       |                                                  |                                                                                                                                                                                                                                                                                                                                              |                                                                                                                                                                                                                                        |
|  "Category_Humanoid"  |             "dtAttackDamageOverride"             |                                                                       Override your mob's damage with specific damage values by type. If you don't want to override the original mob's damage, you can set this block to null, as in `"dtAttackDamageOverride" : null`                                                                       |                                  {"fDamage" : 0, "fBlunt" : 0, "fSlash" : 0, "fPierce" : 0, "fChop" : 0, "fPickaxe" : 0, "fFire" : 25, "fFrost" : 25, "fLightning" : 25, "fPoison" : 0, "fSpirit" : 0}                                 |
|  "Category_Humanoid"  |           "fAttackDamageTotalOverride"           |                    Override your mob's damage with a fixed total number. Can be null if you do not wish to override. The original damage type proportions of their attacks will be "squished" down to the given total. Note that this means zero-damage attacks/mobs will still do zero damage (zero * any number = zero).                   |                                                                                                                   100                                                                                                                  |
|  "Category_Humanoid"  |            "sAttackProjectileOverride"           |                                                                                                     Override your mob's projectile attacks with a new projectile. See `ConfigGuide_Projectiles.txt` for a list of stock game projectiles.                                                                                                    |                                                                                                        "Imp_fireball_projectile"                                                                                                       |
|  "Category_Humanoid"  |                  "aDefaultItems"                 |                                                                                                                                         All items in this list will appear on every copy of your mob.                                                                                                                                        |                                                                                                     ["troll_punch", "troll_throw"]                                                                                                     |
|  "Category_Humanoid"  | "aRandomWeapon", "aRandomArmor", "aRandomShield" |                                       One item from each of these lists will appear on each copy of your mob. You can use an empty entry `""` to represent a chance that nothing in a slot gets picked. For instance, the example to the right would cause your mobs to have a shield 1/3 of the time.                                       |                                                                                                        ["ShieldSilver" , "", ""]                                                                                                       |
|  "Category_Humanoid"  |                  "aaRandomSets"                  |  A list of lists. One of the sets will be picked at random for each copy of your mob, and every item in that list will be added to that mob. The syntax on this one can be messy so I suggest you export a monster like Troll that has sets so you can copy-paste the existing code and tweak it instead of trying to write it from scratch. |                                                   [{"aSetItems" : [ "troll_punch", "troll_groundslam", "troll_throw"]}, {"aSetItems" : [ "troll_log_swing_v", "troll_log_swing_h"]}]                                                   |
|   "Category_BaseAI"   |                   "fViewRange"                   |                                                                                                                                                       How far your mob can see targets.                                                                                                                                                      |                                                                                                                   30                                                                                                                   |
|   "Category_BaseAI"   |                   "fViewAngle"                   |                                                                                                                   The angle of vision for your mob. Max is 360, at which point your mob can see in all directions at once.                                                                                                                   |                                                                                                                   90                                                                                                                   |
|   "Category_BaseAI"   |                   "fHearRange"                   |                                                                                                                      How far your mob can hear sounds? Not sure what this does, as it seems to be 9999 for all monsters.                                                                                                                     |                                                                                                                  9999                                                                                                                  |
|   "Category_BaseAI"   |                 "sPathAgentType"                 | Not certain what this does, seems to be based on the size of the mob. If you make a small mob big, you should use a bigger pathing agent.  Valid entries: `"Humanoid"`, `"TrollSize"`, `"HugeSize"`, `"HorseSize"`, `"HumanoidNoSwim"`, `"HumanoidAvoidWater"`, `"Fish"`, `"Wolf"`, `"BigFish"`, `"GoblinBruteSize"`, `"HumanoidBigNoSwim"`. |                                                                                                               "TrollSize"                                                                                                              |
|   "Category_BaseAI"   |              "fRandomCircleInterval"             |                                                                                                                                                               Function unknown.                                                                                                                                                              |                                                                                                                    2                                                                                                                   |
|   "Category_BaseAI"   |               "fRandomMoveInterval"              |                                                                                                                                                               Function unknown.                                                                                                                                                              |                                                                                                                   30                                                                                                                   |
|   "Category_BaseAI"   |                "fRandomMoveRange"                |                                                                                                                                                               Function unknown.                                                                                                                                                              |                                                                                                                   10                                                                                                                   |
|   "Category_BaseAI"   |                   "bAvoidFire"                   |                                                                                                                                                               Function unknown.                                                                                                                                                              |                                                                                                                  false                                                                                                                 |
|   "Category_BaseAI"   |                  "bAfraidOfFire"                 |                                                                                                                                                               Function unknown.                                                                                                                                                              |                                                                                                                  false                                                                                                                 |
|   "Category_BaseAI"   |                   "bAvoidWater"                  |                                                                                                                                                               Function unknown.                                                                                                                                                              |                                                                                                                  true                                                                                                                  |
|   "Category_BaseAI"   |                  "sSpawnMessage"                 |                                                                                                                                       HUD message that appears for nearby players when this mob spawns.                                                                                                                                      |                                                                                                        "The Elder has awoken..."                                                                                                       |
|   "Category_BaseAI"   |                  "sDeathMessage"                 |                                                                                                                                        HUD message that appears for nearby players when this mob dies.                                                                                                                                       |                                                                                                       "The Elder has uh, died..."                                                                                                      |


**Writing out this list took longer than expected! For now, you can learn a lot be exporting lots of Monsters and looking at how they are configured. You can also ask for help in the Valheim Modding Discord at https://discord.gg/RBq2mzeu4z . I will finish this table for the next version :)**